@echo off
title Engine Aggressiveness Statistics Tool V5.61 by Stefan Pohl, www.sp-cc.de

REM **************************************************************************************
REM *** special hardcoded shortwin_movelimit here, change to other values, if you want ***
REM *** set it to 0, to deactivate the hardcode-override 
set /A hardlimit=60
REM **************************************************************************************

@echo ***************************************************************
@echo ***        Engine Aggressiveness Statistics Tool V5.61      ***
@echo ***           (C) 2024, Stefan Pohl, www.sp-cc.de           ***
@echo ***************************************************************
@echo ***     This tool uses pgn-extract (C) David J. Barnes      ***
@echo ***        and some pgn-tools by (C) Norman Pollock         ***
@echo ***************************************************************
@echo ***************************************************************

@echo Enter the name of your games pgn-file (without .pgn ending!) The file must be in the working folder!
@echo off

set /p filename=
set gamebase=%filename%.pgn

CALL :StartTimer

del sacgames_1_pawns.pgn 2>nul
del sacgames_2_pawns.pgn 2>nul
del sacgames_3_pawns.pgn 2>nul
del sacgames_4_pawns.pgn 2>nul
del sacgames_5_pawns.pgn 2>nul
del sacgames_queensacs.pgn 2>nul
del shortwins.pgn 2>nul

cd ./bin
del statistics_EAS_rating_work.txt 2>nul
del statistics_EAS_rating_work2.txt 2>nul
del newsource.pgn 2>nul
del newsource_onlywins.pgn 2>nul
del enginegames.pgn 2>nul
del collect_sacgames_1.pgn 2>nul
del collect_sacgames_2.pgn 2>nul
del collect_sacgames_3.pgn 2>nul
del collect_sacgames_4.pgn 2>nul
del collect_sacgames_5.pgn 2>nul
del collect_sacgames_9.pgn 2>nul
del collect_shorts.pgn 2>nul

REM *** EAS single stats ***
del eas_singlestat_A.txt 2>nul
del eas_singlestat_B.txt 2>nul
del eas_singlestat_C.txt 2>nul
del eas_singlestat_D.txt 2>nul
del eas_singlestat_E.txt 2>nul
del eas_singlestat_F.txt 2>nul
del eas_singlestat_A_sorted.txt 2>nul
del eas_singlestat_B_sorted.txt 2>nul
del eas_singlestat_C_sorted.txt 2>nul
del eas_singlestat_D_sorted.txt 2>nul
del eas_singlestat_E_sorted.txt 2>nul
del eas_singlestat_F_sorted.txt 2>nul
del eas_singlestat_out.txt 2>nul


@echo *********************************************************************************************
@echo *** Check games, fix result-tags and delete all comments for faster computing
pgn-extract --quiet --fixresulttags -C -N -V --plycount ../%gamebase% --output newsource.pgn > NUL

@echo *********************************************************************************************
@echo *** Filter won games out of the source.pgn-file for calculating average game length of wins
pgn-extract --quiet -Tr1-0 -Tr0-1 newsource.pgn --output newsource_onlywins.pgn > NUL

REM *** Calculate average length of all won games in the source.pgn file, later needed for setting
REM *** the intervals of the length of the short games
CALL :moveaverage newsource_onlywins.pgn
set /A avg_length_all_wins=%ma_moveaverage%

REM *** Calculate movelimit for short wins, depending on the average won game length of
REM *** all games of the source.pgn file
set /A shortwin_movelimit=(%avg_length_all_wins%/5)*5
set /A shortwin_movelimit-=15
if %shortwin_movelimit% LSS 30 set /A shortwin_movelimit=30
if %shortwin_movelimit% GTR 95 set /A shortwin_movelimit=95

REM **************************************************************************************
REM *** special hardcoded shortwin_movelimit here 
if %hardlimit% GTR 0 set /A shortwin_movelimit=%hardlimit%
REM **************************************************************************************

set /A sh_level1=%shortwin_movelimit%
set /A sh_level2=%shortwin_movelimit%-5
set /A sh_level3=%shortwin_movelimit%-10
set /A sh_level4=%shortwin_movelimit%-15
set /A sh_level5=%shortwin_movelimit%-20

@echo *********************************************************************************************
@echo *** Counting number of engines in the pgn-file
REM *** Use Norm Pollock tool nameList to find all engine-names in the pgn-file ***
del outname* 2>nul
nameList newsource.pgn >NUL

REM *** Count number of lines in outname-file (= number of different engines in the source.pgn) ***
set /A numb_engines=0 
for /f %%a in (outname) do set /a numb_engines+=1

REM *** Read the names of all engines and put them in a string-array ***
set /A counter=1
:loop01
set /p engines[%counter%]=<outname
set /A counter=%counter%+1
if %counter% GTR %numb_engines% goto exitloop01
more +1 outname > outnamex 
set /p engines[%counter%]=<outnamex
more +1 outnamex > outname 
set /A counter=%counter%+1
if %counter% GTR %numb_engines% goto exitloop01
goto loop01
:exitloop01
del outname*

@echo *********************************************************************************************
@echo *** Number of engines found in the pgn-file: %numb_engines%
REM *** Start second timer for engine-processing. Needed for estimating the time to finish...
CALL :StartTimer2

REM *** Start of processing all engines, one after another, in a loop ***
set /A engine_counter=1
:loop02

del enginefile 2>nul 
if defined engines[%engine_counter%]  (
call echo "%%engines[%engine_counter%]%%">enginefile
set /p engine=<enginefile
del enginefile 2>nul 
)
@echo XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
@echo *** Processing engine %engine_counter%/%numb_engines% (%engine%)

@echo *********************************************************************************************
@echo *** Copy all games played by engine in several working files...
pgn-extract --quiet -Tp%engine% newsource.pgn --output enginegames.pgn > NUL
pgn-extract --quiet -Tr1/2-1/2 enginegames.pgn --output enginedraws.pgn > NUL
pgn-extract --quiet -Tr1-0 -Tw%engine% enginegames.pgn --output whitewins.pgn > NUL
pgn-extract --quiet -Tr0-1 -Tb%engine% enginegames.pgn --output blackwins.pgn > NUL
del allwins.pgn 2>nul
copy whitewins.pgn+blackwins.pgn allwins.pgn > NUL

@echo *********************************************************************************************
@echo|set /p="*** Counting number of wins: "
REM *** Calculate average length of all wins of the engine
CALL :moveaverage allwins.pgn
set /A avg_length_eng_wins=%ma_moveaverage%

if %ma_moveaverage% LEQ 9 (
@echo 00%ma_moveaverage% %engine% >>eas_singlestat_E.txt
goto moveavg_format)
if %ma_moveaverage% LEQ 99 (
@echo 0%ma_moveaverage% %engine% >>eas_singlestat_E.txt
goto moveavg_format)
if %ma_moveaverage% LEQ 999 (
@echo %ma_moveaverage% %engine% >>eas_singlestat_E.txt
goto moveavg_format)
:moveavg_format

REM *** Count number of won games by the engine ***
CALL :countgames allwins.pgn
set /A numb_wins=%c_numb_pgn%
@echo|set /p=%numb_wins%

@echo|set /p="...draws: "

REM *** Count number of draw games by the engine ***
CALL :countgames enginedraws.pgn
set /A numb_draws=%c_numb_pgn%
@echo %numb_draws%

REM *** If overall number of wins/draws is too small, print a warning behind the engine name in the
REM *** first ratinglist...
set /A warning=0
if %numb_wins% LSS 50 set /A warning=1
if %numb_draws% LSS 30 set /A warning=1
if %warning% GTR 0 @echo XXXXX WARNING: Not enough games, EAS-score not reliable [50+ wins and 30+ draws needed] XXXXX

set /A engine_eas=0
set /A eas_bad_draws=0
set /A eas_short_wins=0
set /A eas_sacs=0

REM *** Searching for "bad draws": Draws, which ended before endgame
REM *** or draws, where engine had more material than the opponent (1 pawns or more) in the game
@echo *********************************************************************************************
@echo|set /p="*** Searching for bad draws"

REM *** Find draws, which ended before endgame was reached (=bad draw)
pgn-extract --quiet -zno_endgame_draws enginedraws.pgn --output results.pgn >NUL
pgn-extract --quiet -cresults.pgn -D -obad_draws2.pgn enginedraws.pgn > NUL

REM *** Find all draws, were engine had material advantage (=bad draw)
pgn-extract --quiet -Tw%engine% -y1_pawnsac_black enginedraws.pgn -abad_draws2.pgn >NUL
pgn-extract --quiet -Tb%engine% -y1_pawnsac_white enginedraws.pgn -abad_draws2.pgn >NUL

REM *** Find draws, which ended before endgame, but were engine had material disadvantage (=no bad draw, delete these games)
pgn-extract --quiet -Tw%engine% -y1_pawnsac_white bad_draws2.pgn --output savedgames.pgn >NUL
pgn-extract --quiet -Tb%engine% -y1_pawnsac_black bad_draws2.pgn -asavedgames.pgn >NUL
pgn-extract --quiet -csavedgames.pgn -D -obad_draws.pgn bad_draws2.pgn > NUL

REM *** Delete possible double games
pgn-extract --quiet -D bad_draws.pgn --output bad_draws_final.pgn > NUL

REM *** Count number of bad draw games by the engine ***
CALL :countgames bad_draws_final.pgn
set /A numb_bad_draws=%c_numb_pgn%
CALL :percent %numb_draws% %numb_bad_draws%
set percent_bad_draws=%percent%

@echo %percent% %engine% >>eas_singlestat_F.txt

REM *** EAS-score is calculated out of the good draws:
REM *** (Percent of good draws exp3)/2500 and
REM *** this result exp2...

set /A numb_good_draws=%numb_draws%-%numb_bad_draws%
CALL :percent %numb_draws% %numb_good_draws%
set /A gd=%percentx100%/100
set /A temp_bd=(%gd%*%gd%*%gd%)/2500
set /A temp_bd=%temp_bd%*%temp_bd%

set /A engine_eas+=%temp_bd%
set /A eas_bad_draws=%engine_eas%


@echo|set /p="...short wins"
pgn-extract --quiet -bu%sh_level1% allwins.pgn --output short_mvs_wins.pgn > NUL
pgn-extract --quiet short_mvs_wins.pgn -acollect_shorts.pgn > NUL

REM *** Old 40 move limit is now sh_level5, 45 is sh_level4, 50 is sh_level3, 55 is sh_level2
REM *** 60 is sh_level1, but there is no need for renaming

set /A eas_singlestat_C_temp=0
REM *** Count number of wins of the engine, not longer than 40 moves ***
pgn-extract --quiet -bu%sh_level5% short_mvs_wins.pgn --output results.pgn > NUL
CALL :countgames results.pgn
set /A numb_shortwins=%c_numb_pgn%
set /A won_40_mvs=%numb_shortwins%
CALL :percent %numb_wins% %numb_shortwins%
set perc_40mvs=%percent%
set /A engine_eas+=%percentx100%*45
set /A eas_singlestat_C_temp=%numb_shortwins%

REM *** Count number of wins of the engine, not longer than 45 moves ***
pgn-extract --quiet -bu%sh_level4% short_mvs_wins.pgn --output results.pgn > NUL
CALL :countgames results.pgn
set /A numb_shortwins=%c_numb_pgn%
set /A won_45_mvs=%numb_shortwins%
set /A numb_shortwins=%numb_shortwins%-%won_40_mvs%
CALL :percent %numb_wins% %numb_shortwins%
set perc_45mvs=%percent%
set /A engine_eas+=%percentx100%*27
set /A eas_singlestat_C_temp+=%numb_shortwins%

CALL :percent %numb_wins% %eas_singlestat_C_temp%
@echo %percent% %engine% >>eas_singlestat_C.txt

REM *** Count number of wins of the engine, not longer than 50 moves ***
pgn-extract --quiet -bu%sh_level3% short_mvs_wins.pgn --output results.pgn > NUL
CALL :countgames results.pgn
set /A numb_shortwins=%c_numb_pgn%
set /A won_50_mvs=%numb_shortwins%
set /A numb_shortwins=%numb_shortwins%-%won_45_mvs%
CALL :percent %numb_wins% %numb_shortwins%
set perc_50mvs=%percent%
set /A engine_eas+=%percentx100%*18

REM *** Count number of wins of the engine, not longer than 55 moves ***
pgn-extract --quiet -bu%sh_level2% short_mvs_wins.pgn --output results.pgn > NUL
CALL :countgames results.pgn
set /A numb_shortwins=%c_numb_pgn%
set /A won_55_mvs=%numb_shortwins%
set /A numb_shortwins=%numb_shortwins%-%won_50_mvs%
CALL :percent %numb_wins% %numb_shortwins%
set perc_55mvs=%percent%
set /A engine_eas+=%percentx100%*12

REM *** Count number of wins of the engine, not longer than 60 moves ***
CALL :countgames short_mvs_wins.pgn
set /A numb_shortwins=%c_numb_pgn%

REM *** For the ratinglist-output, build the percent-number of all short wins
CALL :percent %numb_wins% %numb_shortwins%
set percent_all_shorts=%percent%

@echo %percent% %engine% >>eas_singlestat_D.txt

set /A won_60_mvs=%numb_shortwins%
set /A numb_shortwins=%numb_shortwins%-%won_55_mvs%
CALL :percent %numb_wins% %numb_shortwins%
set perc_60mvs=%percent%
set /A engine_eas+=%percentx100%*8


REM *** Add EAS Bonuspoints if the average length of the won games by the engine is
REM *** lower than the average length of all won games in the source.pgn
REM *** Otherwise sub some ESA points.
set /A sub_eas=0
set /A eas_short_wins=%engine_eas%-%eas_bad_draws%
set /A check_average=%avg_length_all_wins%-%avg_length_eng_wins%
if %check_average% GTR 0 set /A engine_eas+=(%check_average%*3000)
if %check_average% LSS 0 set /A sub_eas-=(%check_average%*1000)
if %sub_eas% GTR %eas_short_wins% set /A sub_eas=%eas_short_wins%
if %sub_eas% GTR 0 set /A engine_eas-=%sub_eas%

set /A eas_short_wins=%engine_eas%-%eas_bad_draws%


REM *** Search for the won games with sacrifices, played by the engine ***

@echo|set /p="...1+ sacs white"
pgn-extract --quiet -y1_pawnsac_white whitewins.pgn --output results_opt1.pgn >NUL
del whitewins.pgn 2>nul
copy results_opt1.pgn whitewins.pgn > NUL
@echo|set /p="...1+ sacs black"
pgn-extract --quiet -y1_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt1.pgn blacksacs.pgn >NUL

del blackwins.pgn 2>nul
copy blacksacs.pgn blackwins.pgn > NUL

@echo|set /p="...2-5+ sacs"
pgn-extract --quiet -y2_pawnsac_white whitewins.pgn --output results_opt2.pgn >NUL
pgn-extract --quiet -y2_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
copy results_opt2.pgn whitewins.pgn > NUL
copy blacksacs.pgn blackwins.pgn > NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt2.pgn blacksacs.pgn >NUL
fsutil file createnew blacksacs.pgn 0 >NUL

pgn-extract --quiet -y3_pawnsac_white whitewins.pgn --output results_opt3.pgn >NUL
pgn-extract --quiet -y3_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
copy results_opt3.pgn whitewins.pgn > NUL
copy blacksacs.pgn blackwins.pgn > NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt3.pgn blacksacs.pgn >NUL
fsutil file createnew blacksacs.pgn 0 >NUL

pgn-extract --quiet -y4_pawnsac_white whitewins.pgn --output results_opt4.pgn >NUL
pgn-extract --quiet -y4_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
copy results_opt4.pgn whitewins.pgn > NUL
copy blacksacs.pgn blackwins.pgn > NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt4.pgn blacksacs.pgn >NUL
fsutil file createnew blacksacs.pgn 0 >NUL

pgn-extract --quiet -y5_pawnsac_white whitewins.pgn --output results_opt5.pgn >NUL
pgn-extract --quiet -y5_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
copy results_opt5.pgn whitewins.pgn > NUL
copy blacksacs.pgn blackwins.pgn > NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt5.pgn blacksacs.pgn >NUL

pgn-extract --quiet -yqueensac_white whitewins.pgn --output results_opt9.pgn >NUL
pgn-extract --quiet -yqueensac_black blackwins.pgn --output blacksacs.pgn >NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt9.pgn blacksacs.pgn >NUL

REM *** Delete games in the gamebases with lower sacs, which already occur in higher sacs files...***
pgn-extract --quiet -D results_opt9.pgn --output unique_opt9.pgn >NUL
pgn-extract --quiet -cresults_opt9.pgn -D -ounique_opt5.pgn results_opt5.pgn > NUL
pgn-extract --quiet -cresults_opt5.pgn -D -ounique_opt4.pgn results_opt4.pgn > NUL
pgn-extract --quiet -cresults_opt4.pgn -D -ounique_opt3.pgn results_opt3.pgn > NUL
pgn-extract --quiet -cresults_opt3.pgn -D -ounique_opt2.pgn results_opt2.pgn > NUL
pgn-extract --quiet -cresults_opt2.pgn -D -ounique_opt1.pgn results_opt1.pgn > NUL

REM *** Count the number of games in the result bases...***

CALL :countgames unique_opt9.pgn
set /A numb_opt9=%c_numb_pgn%
pgn-extract --quiet unique_opt9.pgn -acollect_sacgames_9.pgn > NUL
CALL :countgames unique_opt5.pgn
set /A numb_opt5=%c_numb_pgn%
pgn-extract --quiet unique_opt5.pgn -acollect_sacgames_5.pgn > NUL
CALL :countgames unique_opt4.pgn
set /A numb_opt4=%c_numb_pgn%
pgn-extract --quiet unique_opt4.pgn -acollect_sacgames_4.pgn > NUL
CALL :countgames unique_opt3.pgn
set /A numb_opt3=%c_numb_pgn%
pgn-extract --quiet unique_opt3.pgn -acollect_sacgames_3.pgn > NUL
CALL :countgames unique_opt2.pgn
set /A numb_opt2=%c_numb_pgn%
pgn-extract --quiet unique_opt2.pgn -acollect_sacgames_2.pgn > NUL
CALL :countgames unique_opt1.pgn
set /A numb_opt1=%c_numb_pgn%
pgn-extract --quiet unique_opt1.pgn -acollect_sacgames_1.pgn > NUL

REM *** Count the number off all sacs found (add all 6 sac-numbers) ***
set /A numb_sum=%numb_opt1%+%numb_opt2%+%numb_opt3%+%numb_opt4%+%numb_opt5%+%numb_opt9%

REM *** For the ratinglist-output, build the percent-number of all sac-games 
CALL :percent %numb_wins% %numb_sum%
set percent_all_sacs=%percent%

@echo %percent% %engine% >>eas_singlestat_B.txt


REM *** Count the sacs-percents of won games (higher sacs = bonus) for a engine-sac ratinglist ***
CALL :percent %numb_wins% %numb_opt1%
set perc_sac1=%percent%
set /A engine_eas+=%percentx100%*5

CALL :percent %numb_wins% %numb_opt2%
set perc_sac2=%percent%
set /A engine_eas+=%percentx100%*20

CALL :percent %numb_wins% %numb_opt3%
set perc_sac3=%percent%
set /A engine_eas+=%percentx100%*50

CALL :percent %numb_wins% %numb_opt4%
set perc_sac4=%percent%
set /A engine_eas+=%percentx100%*100

CALL :percent %numb_wins% %numb_opt5%
set perc_sac5=%percent%
set /A engine_eas+=%percentx100%*200

CALL :percent %numb_wins% %numb_opt9%
set perc_sac9=%percent%
set /A engine_eas+=%percentx100%*400

set /A eas_singlestat_A_temp=%numb_opt3%+%numb_opt4%+%numb_opt5%+%numb_opt9%
CALL :percent %numb_wins% %eas_singlestat_A_temp%
@echo %percent% %engine% >>eas_singlestat_A.txt


set /A eas_sacs=%engine_eas%-%eas_short_wins%-%eas_bad_draws%

REM *** For a valid sorting of the EAS-scores, they have to be formatted... ***
CALL :format_eas %engine_eas%
set engine_eas=%formatted%

REM *** Format number of wins
CALL :format_wins %numb_wins%
set winform=%formatted%

REM *** Format average length of won games by engine
CALL :format_engwins %avg_length_eng_wins%
set avg_length_eng_wins=%formatted%

REM *** Format average length of won games by engine
CALL :format_engwins %avg_length_all_wins%
set avg_length_all_wins=%formatted%

REM *** write the score and the engine-name in a file, but first remove the " " from the engine-name
for /f "useback tokens=*" %%a in ('%engine%') do set engine=%%~a

REM *** If overall number of wins/draws is too small, print a warning behind the engine name in the first ratinglist...
if %warning% GTR 0 @echo  %engine_eas%  %percent_all_sacs%  %percent_all_shorts%  %percent_bad_draws%  %avg_length_eng_wins%   %engine%   XXXXX WARNING: Not enough games, EAS-score not reliable [50+ wins and 30+ draws needed] XXXXX >>statistics_EAS_rating_work.txt
if %warning% EQU 0 @echo  %engine_eas%  %percent_all_sacs%  %percent_all_shorts%  %percent_bad_draws%  %avg_length_eng_wins%   %engine% >>statistics_EAS_rating_work.txt

REM *** Debug-line: @echo  %engine_eas%  %percent_all_sacs%  %percent_all_shorts%  %percent_bad_draws%  %avg_length_eng_wins%  [%eas_sacs%  %eas_short_wins% (-%sub_eas%)  %eas_bad_draws%]  %engine% >>statistics_EAS_rating_work.txt
REM *** Original-line: @echo  %engine_eas%  %percent_all_sacs%  %percent_all_shorts%  %percent_bad_draws%  %avg_length_eng_wins%   %engine% >>statistics_EAS_rating_work.txt

@echo  %engine_eas%   %winform%  %avg_length_eng_wins%   %percent_all_sacs% =[%perc_sac9% + %perc_sac5% + %perc_sac4% + %perc_sac3% + %perc_sac2% + %perc_sac1%]    %percent_all_shorts% = [%perc_40mvs% + %perc_45mvs% + %perc_50mvs% + %perc_55mvs% + %perc_60mvs%]  %percent_bad_draws%   %engine% >>statistics_EAS_rating_work2.txt

cd ../

REM *** All engines done? then exit, otherwise jump back but show estimated time to finish first ***
@echo ...done
@echo *********************************************************************************************
CALL :StopTimer2
CALL :EstimatedTime
@echo *********************************************************************************************

set /A engine_counter=%engine_counter%+1
if %engine_counter% GTR %numb_engines% goto exitloop02
cd ./bin
goto loop02
:exitloop02

@echo *********************************************************************************************
@echo *** Done!  Building the final ratinglist-file and the pgn-file with shorts and sac games
@echo *********************************************************************************************

REM *** Build the EAS-ratinglist file ***
del statistics_EAS_ratinglist.txt 2>nul
@echo ***************************************************************************** >statistics_EAS_ratinglist.txt
@echo *** Engine Aggressiveness Tool V5.61 Score points Ratinglist >>statistics_EAS_ratinglist.txt
@echo ***************************************************************************** >>statistics_EAS_ratinglist.txt
@echo *** Meanwhile, the scoring-system of the EAS-Tool got really complex, so >>statistics_EAS_ratinglist.txt
@echo *** please check out the ReadMe-file, where you find the explanation... >>statistics_EAS_ratinglist.txt
@echo ***************************************************************************** >>statistics_EAS_ratinglist.txt
@echo *** Evaluated file: %gamebase% >>statistics_EAS_ratinglist.txt
@echo ***************************************************************************** >>statistics_EAS_ratinglist.txt
@echo                                  bad  avg.win >>statistics_EAS_ratinglist.txt
@echo Rank  EAS-Score  sacs   shorts  draws  moves  Engine/player >>statistics_EAS_ratinglist.txt
@echo ------------------------------------------------------------------->>statistics_EAS_ratinglist.txt
cd ./bin
del listnumbs.txt 2>nul
del listnumbs2.txt 2>nul
del list2numbs.txt 2>nul
del list2numbs2.txt 2>nul

REM *** Sorting lists by eas-score, then add rank-numbers to the EAS-ratinglist ***
sort /r statistics_EAS_rating_work.txt >>listnumbs.txt
sort /r statistics_EAS_rating_work2.txt >>list2numbs.txt

set /A listnumber=0
:listloop01
set /A listnumber+=1
set /p nextengine=<listnumbs.txt
more +1 listnumbs.txt > listnumbs2.txt
del listnumbs.txt 2>nul
rename listnumbs2.txt listnumbs.txt

REM *** Format engine-number for output
CALL :format_engines %listnumber%
@echo %formatted%  %nextengine% >>../statistics_EAS_ratinglist.txt

if %listnumber% LSS %numb_engines% goto listloop01
@echo ------------------------------------------------------------------->>../statistics_EAS_ratinglist.txt
@echo *** Average length of all won games:    %avg_length_all_wins% moves>>../statistics_EAS_ratinglist.txt
@echo *** Engine gets bonuspoints, if its avg. won games length is shorter>>../statistics_EAS_ratinglist.txt
@echo *** Engine gets maluspoints, if its avg. won games length is longer>>../statistics_EAS_ratinglist.txt

REM *** EAS single stats subroutine, makes only sense, if at least 3 engines were computed
if %numb_engines% GTR 2 (
CALL :eas_single_stats
more eas_singlestat_out.txt >>../statistics_EAS_ratinglist.txt
)

@echo ***************************************************************************** >>../statistics_EAS_ratinglist.txt
@echo ***************************************************************************** >>../statistics_EAS_ratinglist.txt
@echo ***************************************************************************** >>../statistics_EAS_ratinglist.txt
@echo *** 2nd Ratinglist with more stats in percent-values ************************ >>../statistics_EAS_ratinglist.txt
@echo ***************************************************************************** >>../statistics_EAS_ratinglist.txt
@echo *** Average length of all won games: %avg_length_all_wins% moves >>../statistics_EAS_ratinglist.txt
@echo *** Calculated limit for short wins giving EAS-points: %shortwin_movelimit% moves >>../statistics_EAS_ratinglist.txt
@echo ***************************************************************************** >>../statistics_EAS_ratinglist.txt
@echo                        avg.win                                                                                                                          bad   >>../statistics_EAS_ratinglist.txt
@echo Rank  EAS-Score   wins  moves   sacs    sacsQ    sacs5+   sacs4    sacs3    sacs2    sacs1    all shorts short%sh_level5%  short%sh_level4%  short%sh_level3%  short%sh_level2%  short%sh_level1%   draws    Engine/player>>../statistics_EAS_ratinglist.txt
@echo ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------->>../statistics_EAS_ratinglist.txt

REM *** Build second ratinglist with all numbers of wins/sacs/short-wins ***
set /A listnumber=0
:list2loop01
set /A listnumber+=1
set /p nextengine=<list2numbs.txt
more +1 list2numbs.txt > list2numbs2.txt
del list2numbs.txt 2>nul
rename list2numbs2.txt list2numbs.txt

REM *** Format engine-number for output
CALL :format_engines %listnumber%
@echo %formatted%  %nextengine% >>../statistics_EAS_ratinglist.txt

if %listnumber% LSS %numb_engines% goto list2loop01

del listnumbs.txt 2>nul
del listnumbs2.txt 2>nul
del list2numbs.txt 2>nul
del list2numbs2.txt 2>nul
del statistics_EAS_rating_work.txt 2>nul
del statistics_EAS_rating_work2.txt 2>nul

REM *** delete all working files ***
del results.pgn 2>nul
del results_opt1.pgn 2>nul
del results_opt2.pgn 2>nul 
del results_opt3.pgn 2>nul 
del results_opt4.pgn 2>nul 
del results_opt5.pgn 2>nul 
del results_opt9.pgn 2>nul 
del unique_opt1.pgn 2>nul 
del unique_opt2.pgn 2>nul 
del unique_opt3.pgn 2>nul 
del unique_opt4.pgn 2>nul 
del unique_opt5.pgn 2>nul 
del unique_opt9.pgn 2>nul 
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
del blacksacs.pgn 2>nul
del allwins.pgn 2>nul
del short_mvs_wins.pgn 2>nul
del bad_draws.pgn 2>nul
del bad_draws2.pgn 2>nul
del bad_draws_final.pgn 2>nul
del savedgames.pgn 2>nul
del enginegames.pgn 2>nul
del enginedraws.pgn 2>nul
del newsource.pgn 2>nul
del newsource_onlywins.pgn 2>nul
REM *** EAS single stats ***
del eas_singlestat_A.txt 2>nul
del eas_singlestat_B.txt 2>nul
del eas_singlestat_C.txt 2>nul
del eas_singlestat_D.txt 2>nul
del eas_singlestat_E.txt 2>nul
del eas_singlestat_F.txt 2>nul
del eas_singlestat_A_sorted.txt 2>nul
del eas_singlestat_B_sorted.txt 2>nul
del eas_singlestat_C_sorted.txt 2>nul
del eas_singlestat_D_sorted.txt 2>nul
del eas_singlestat_E_sorted.txt 2>nul
del eas_singlestat_F_sorted.txt 2>nul
del eas_singlestat_out.txt 2>nul

REM *** Build output pgn-files (sacs, sorted by value in different files, no doubles)
pgn-extract --quiet -D collect_sacgames_9.pgn --output ../sacgames_queensacs.pgn > NUL
pgn-extract --quiet -D collect_sacgames_5.pgn --output ../sacgames_5_pawns.pgn > NUL
pgn-extract --quiet -D collect_sacgames_4.pgn --output ../sacgames_4_pawns.pgn > NUL
pgn-extract --quiet -D collect_sacgames_3.pgn --output ../sacgames_3_pawns.pgn > NUL
pgn-extract --quiet -D collect_sacgames_2.pgn --output ../sacgames_2_pawns.pgn > NUL
pgn-extract --quiet -D collect_sacgames_1.pgn --output ../sacgames_1_pawns.pgn > NUL
pgn-extract --quiet -D collect_shorts.pgn --output ../shortwins.pgn > NUL
del collect_sacgames_1.pgn 2>nul
del collect_sacgames_2.pgn 2>nul
del collect_sacgames_3.pgn 2>nul
del collect_sacgames_4.pgn 2>nul
del collect_sacgames_5.pgn 2>nul
del collect_sacgames_9.pgn 2>nul
del collect_shorts.pgn 2>nul

cd ../

@echo **************************************************** >>statistics_EAS_ratinglist.txt
@echo *** EAS-tool (C) 2024 Stefan Pohl (www.sp-cc.de) *** >>statistics_EAS_ratinglist.txt
@echo **************************************************** >>statistics_EAS_ratinglist.txt

@echo ****************
@echo *** Finished ***
@echo ****************
@echo *********************************************************************************************
@echo *** See the EAS Scores of the engines/players in the statistics_EAS_ratinglist.txt file
@echo *********************************************************************************************
@echo *** All sac-games are stored in the sacgames - pgn-files...(without comments and any double 
@echo *** games). All shortwins (up to %shortwin_movelimit% moves) are stored in the shortwins.pgn file
@echo *** Mention: There can be double games (a shortwin game can be a sac-game, too)
@echo *********************************************************************************************
CALL :StopTimer
CALL :DisplayTimerResult
@echo *********************************************************************************************
:error
CALL ./bin/playwav %windir%/media/alarm02.wav
pause
goto :EOF

:eas_single_stats
sort /r eas_singlestat_A.txt >>eas_singlestat_A_sorted.txt
sort /r eas_singlestat_B.txt >>eas_singlestat_B_sorted.txt
sort /r eas_singlestat_C.txt >>eas_singlestat_C_sorted.txt
sort /r eas_singlestat_D.txt >>eas_singlestat_D_sorted.txt
sort eas_singlestat_E.txt >>eas_singlestat_E_sorted.txt
sort eas_singlestat_F.txt >>eas_singlestat_F_sorted.txt

more eas_singlestat_A_sorted.txt > eas_gold_A_output.txt
more +1 eas_singlestat_A_sorted.txt > eas_silver_A_output.txt
more +2 eas_singlestat_A_sorted.txt > eas_bronze_A_output.txt
more +3 eas_singlestat_A_sorted.txt > eas_fourth_A_output.txt
more +4 eas_singlestat_A_sorted.txt > eas_fifth_A_output.txt
set /p eas_A_goldmedal=<eas_gold_A_output.txt
set /p eas_A_silvermedal=<eas_silver_A_output.txt
set /p eas_A_bronzemedal=<eas_bronze_A_output.txt
set /p eas_A_fourthmedal=<eas_fourth_A_output.txt
set /p eas_A_fifthmedal=<eas_fifth_A_output.txt
del eas_gold_A_output.txt 2>nul
del eas_silver_A_output.txt 2>nul
del eas_bronze_A_output.txt 2>nul
del eas_fourth_A_output.txt 2>nul
del eas_fifth_A_output.txt 2>nul
set eas_A_goldmedal=%eas_A_goldmedal:"=%
set eas_A_silvermedal=%eas_A_silvermedal:"=%
set eas_A_bronzemedal=%eas_A_bronzemedal:"=%
set eas_A_fourthmedal=%eas_A_fourthmedal:"=%
set eas_A_fifthmedal=%eas_A_fifthmedal:"=%

more eas_singlestat_B_sorted.txt > eas_gold_B_output.txt
more +1 eas_singlestat_B_sorted.txt > eas_silver_B_output.txt
more +2 eas_singlestat_B_sorted.txt > eas_bronze_B_output.txt
more +3 eas_singlestat_B_sorted.txt > eas_fourth_B_output.txt
more +4 eas_singlestat_B_sorted.txt > eas_fifth_B_output.txt
set /p eas_B_goldmedal=<eas_gold_B_output.txt
set /p eas_B_silvermedal=<eas_silver_B_output.txt
set /p eas_B_bronzemedal=<eas_bronze_B_output.txt
set /p eas_B_fourthmedal=<eas_fourth_B_output.txt
set /p eas_B_fifthmedal=<eas_fifth_B_output.txt
del eas_gold_B_output.txt 2>nul
del eas_silver_B_output.txt 2>nul
del eas_bronze_B_output.txt 2>nul
del eas_fourth_B_output.txt 2>nul
del eas_fifth_B_output.txt 2>nul
set eas_B_goldmedal=%eas_B_goldmedal:"=%
set eas_B_silvermedal=%eas_B_silvermedal:"=%
set eas_B_bronzemedal=%eas_B_bronzemedal:"=%
set eas_B_fourthmedal=%eas_B_fourthmedal:"=%
set eas_B_fifthmedal=%eas_B_fifthmedal:"=%

more eas_singlestat_C_sorted.txt > eas_gold_C_output.txt
more +1 eas_singlestat_C_sorted.txt > eas_silver_C_output.txt
more +2 eas_singlestat_C_sorted.txt > eas_bronze_C_output.txt
more +3 eas_singlestat_C_sorted.txt > eas_fourth_C_output.txt
more +4 eas_singlestat_C_sorted.txt > eas_fifth_C_output.txt
set /p eas_C_goldmedal=<eas_gold_C_output.txt
set /p eas_C_silvermedal=<eas_silver_C_output.txt
set /p eas_C_bronzemedal=<eas_bronze_C_output.txt
set /p eas_C_fourthmedal=<eas_fourth_C_output.txt
set /p eas_C_fifthmedal=<eas_fifth_C_output.txt
del eas_gold_C_output.txt 2>nul
del eas_silver_C_output.txt 2>nul
del eas_bronze_C_output.txt 2>nul
del eas_fourth_C_output.txt 2>nul
del eas_fifth_C_output.txt 2>nul
set eas_C_goldmedal=%eas_C_goldmedal:"=%
set eas_C_silvermedal=%eas_C_silvermedal:"=%
set eas_C_bronzemedal=%eas_C_bronzemedal:"=%
set eas_C_fourthmedal=%eas_C_fourthmedal:"=%
set eas_C_fifthmedal=%eas_C_fifthmedal:"=%

more eas_singlestat_D_sorted.txt > eas_gold_D_output.txt
more +1 eas_singlestat_D_sorted.txt > eas_silver_D_output.txt
more +2 eas_singlestat_D_sorted.txt > eas_bronze_D_output.txt
more +3 eas_singlestat_D_sorted.txt > eas_fourth_D_output.txt
more +4 eas_singlestat_D_sorted.txt > eas_fifth_D_output.txt
set /p eas_D_goldmedal=<eas_gold_D_output.txt
set /p eas_D_silvermedal=<eas_silver_D_output.txt
set /p eas_D_bronzemedal=<eas_bronze_D_output.txt
set /p eas_D_fourthmedal=<eas_fourth_D_output.txt
set /p eas_D_fifthmedal=<eas_fifth_D_output.txt
del eas_gold_D_output.txt 2>nul
del eas_silver_D_output.txt 2>nul
del eas_bronze_D_output.txt 2>nul
del eas_fourth_D_output.txt 2>nul
del eas_fifth_D_output.txt 2>nul
set eas_D_goldmedal=%eas_D_goldmedal:"=%
set eas_D_silvermedal=%eas_D_silvermedal:"=%
set eas_D_bronzemedal=%eas_D_bronzemedal:"=%
set eas_D_fourthmedal=%eas_D_fourthmedal:"=%
set eas_D_fifthmedal=%eas_D_fifthmedal:"=%

more eas_singlestat_E_sorted.txt > eas_gold_E_output.txt
more +1 eas_singlestat_E_sorted.txt > eas_silver_E_output.txt
more +2 eas_singlestat_E_sorted.txt > eas_bronze_E_output.txt
more +3 eas_singlestat_E_sorted.txt > eas_fourth_E_output.txt
more +4 eas_singlestat_E_sorted.txt > eas_fifth_E_output.txt
set /p eas_E_goldmedal=<eas_gold_E_output.txt
set /p eas_E_silvermedal=<eas_silver_E_output.txt
set /p eas_E_bronzemedal=<eas_bronze_E_output.txt
set /p eas_E_fourthmedal=<eas_fourth_E_output.txt
set /p eas_E_fifthmedal=<eas_fifth_E_output.txt
del eas_gold_E_output.txt 2>nul
del eas_silver_E_output.txt 2>nul
del eas_bronze_E_output.txt 2>nul
del eas_fourth_E_output.txt 2>nul
del eas_fifth_E_output.txt 2>nul
set eas_E_goldmedal=%eas_E_goldmedal:"=%
set eas_E_silvermedal=%eas_E_silvermedal:"=%
set eas_E_bronzemedal=%eas_E_bronzemedal:"=%
set eas_E_fourthmedal=%eas_E_fourthmedal:"=%
set eas_E_fifthmedal=%eas_E_fifthmedal:"=%

more eas_singlestat_F_sorted.txt > eas_gold_F_output.txt
more +1 eas_singlestat_F_sorted.txt > eas_silver_F_output.txt
more +2 eas_singlestat_F_sorted.txt > eas_bronze_F_output.txt
more +3 eas_singlestat_F_sorted.txt > eas_fourth_F_output.txt
more +4 eas_singlestat_F_sorted.txt > eas_fifth_F_output.txt
set /p eas_F_goldmedal=<eas_gold_F_output.txt
set /p eas_F_silvermedal=<eas_silver_F_output.txt
set /p eas_F_bronzemedal=<eas_bronze_F_output.txt
set /p eas_F_fourthmedal=<eas_fourth_F_output.txt
set /p eas_F_fifthmedal=<eas_fifth_F_output.txt
del eas_gold_F_output.txt 2>nul
del eas_silver_F_output.txt 2>nul
del eas_bronze_F_output.txt 2>nul
del eas_fourth_F_output.txt 2>nul
del eas_fifth_F_output.txt 2>nul
set eas_F_goldmedal=%eas_F_goldmedal:"=%
set eas_F_silvermedal=%eas_F_silvermedal:"=%
set eas_F_bronzemedal=%eas_F_bronzemedal:"=%
set eas_F_fourthmedal=%eas_F_fourthmedal:"=%
set eas_F_fifthmedal=%eas_F_fifthmedal:"=%

@echo ********************************************************************************************* >eas_singlestat_out.txt
@echo ********************************************************************************************* >>eas_singlestat_out.txt
@echo ********************************************************************************************* >>eas_singlestat_out.txt
@echo *** EAS single-statistics (6 categories, each with Top5 engines): >>eas_singlestat_out.txt
@echo ********************************************************************************************* >>eas_singlestat_out.txt
@echo A: Most high-value sacrifices (3+ pawnunits): [1]:%eas_A_goldmedal%  [2]:%eas_A_silvermedal%  [3]:%eas_A_bronzemedal%  [4]:%eas_A_fourthmedal%  [5]:%eas_A_fifthmedal% >>eas_singlestat_out.txt
@echo B: Most sacrifices overall                  : [1]:%eas_B_goldmedal%  [2]:%eas_B_silvermedal%  [3]:%eas_B_bronzemedal%  [4]:%eas_B_fourthmedal%  [5]:%eas_B_fifthmedal% >>eas_singlestat_out.txt
@echo C: Very short wins (%sh_level4% moves or less)       : [1]:%eas_C_goldmedal%  [2]:%eas_C_silvermedal%  [3]:%eas_C_bronzemedal%  [4]:%eas_C_fourthmedal%  [5]:%eas_C_fifthmedal% >>eas_singlestat_out.txt
@echo D: Most short wins overall                  : [1]:%eas_D_goldmedal%  [2]:%eas_D_silvermedal%  [3]:%eas_D_bronzemedal%  [4]:%eas_D_fourthmedal%  [5]:%eas_D_fifthmedal% >>eas_singlestat_out.txt
@echo E: Average length of all won games          : [1]:%eas_E_goldmedal%  [2]:%eas_E_silvermedal%  [3]:%eas_E_bronzemedal%  [4]:%eas_E_fourthmedal%  [5]:%eas_E_fifthmedal% >>eas_singlestat_out.txt
@echo F: Smallest number of bad draws             : [1]:%eas_F_goldmedal%  [2]:%eas_F_silvermedal%  [3]:%eas_F_bronzemedal%  [4]:%eas_F_fourthmedal%  [5]:%eas_F_fifthmedal% >>eas_singlestat_out.txt
@echo ********************************************************************************************* >>eas_singlestat_out.txt
@echo ********************************************************************************************* >>eas_singlestat_out.txt
exit /B 0

:countgames
set /A c_numb_pgn=0
for /f "tokens=2,* delims= " %%G in ('find /C "[White " %~1') do set /A c_numb_pgn=%%H
exit /B 0

:moveaverage
set ma_gamefile=%~1
CALL :countgames %ma_gamefile%
if %c_numb_pgn% LEQ 0 (
set /A ma_moveaverage=0
EXIT /B 0
)
summary.exe %ma_gamefile% >NUL
for /f "tokens=1-10 delims=., " %%A in ('find "Average" outSummary') do ( 
set ma_avg=%%I >NUL
set ma_rest=%%J > NUL
)
set ma_avg=%ma_avg:~0,-1%
set ma_rest=%ma_rest:~0,-1%
REM *** Round ply-moveaverage up, if necessary
if %ma_rest% GEQ 50 set /A ma_avg+=1
set /A ma_moveaverage=%ma_avg%/2
del outSummary 2>nul
EXIT /B 0

:find_gauntlet
del tempgauntlet 2>nul
more +3 outName3 > tempgauntlet
set /p firstline=<tempgauntlet
set str=%firstline:~0,-8%
set str=%str%##
set str=%str:                ##=##%
set str=%str:        ##=##%
set str=%str:    ##=##%
set str=%str:  ##=##%
set str=%str: ##=##%
set str=%str:##=%
set engine="%str%"
del tempgauntlet 2>nul
del outName* 2>nul
EXIT /B 0

:percent
if %~1 LEQ 0 (
set percent=00.00%%
set /A percentx100=0
EXIT /B 0
)
set /A l_base=((1000000000/%~1)*%~2)
set /A l_percent=%l_base%/1000000
set /A l_rest1=%l_percent%%%10
set /A l_percent=%l_percent%/10
set /A l_secdig_pc=%l_base%/100000
set /A l_rest2=%l_secdig_pc%%%10
set /A l_thirddig_pc=%l_base%/10000
set /A l_rest3=%l_thirddig_pc%%%10

REM *** Round 2nd digit up, if 3rd digit is GEQ 5 and round 1st digit and percent-value up, if necessary
if %l_rest3% GEQ 5 set /A l_rest2+=1
if %l_rest2% GEQ 10 (
  set /A l_rest2-=10
  set /A l_rest1+=1
)
if %l_rest1% GEQ 10 (
  set /A l_rest1-=10
  set /A l_percent+=1
)

if %l_percent% LSS 10 (
  set percent=0%l_percent%.%l_rest1%%l_rest2%%%
) else (
  set percent=%l_percent%.%l_rest1%%l_rest2%%%
)
set /A percentx100=(%l_percent%*100)+(%l_rest1%*10)+%l_rest2%

if %l_percent% GTR 99 (
set percent=100.0%%
set /A percentx100=10000
)
EXIT /B 0

:format_eas
set /A fn_numb=%~1
if %fn_numb% LEQ 9 (
set formatted=      %fn_numb%
goto exitformat_eas)
if %fn_numb% LEQ 99 (
set formatted=     %fn_numb%
goto exitformat_eas)
if %fn_numb% LEQ 999 (
set formatted=    %fn_numb%
goto exitformat_eas)
if %fn_numb% LEQ 9999 (
set formatted=   %fn_numb%
goto exitformat_eas)
if %fn_numb% LEQ 99999 (
set formatted=  %fn_numb%
goto exitformat_eas)
if %fn_numb% LEQ 999999 (
set formatted= %fn_numb%
goto exitformat_eas)
if %fn_numb% LEQ 9999999 (
set formatted=%fn_numb%
goto exitformat_eas)
:exitformat_eas
EXIT /B 0

:format_wins
set /A fn_numb=%~1
if %fn_numb% LEQ 9 (
set formatted=     %fn_numb%
goto exitformat_wins)
if %fn_numb% LEQ 99 (
set formatted=    %fn_numb%
goto exitformat_wins)
if %fn_numb% LEQ 999 (
set formatted=   %fn_numb%
goto exitformat_wins)
if %fn_numb% LEQ 9999 (
set formatted=  %fn_numb%
goto exitformat_wins)
if %fn_numb% LEQ 99999 (
set formatted= %fn_numb%
goto exitformat_wins)
if %fn_numb% LEQ 999999 (
set formatted=%fn_numb%
goto exitformat_eas)
:exitformat_wins
EXIT /B 0

:format_engines
set /A fn_numb=%~1
if %fn_numb% LEQ 9 (
set formatted=   %fn_numb%
goto exitformat_engines)
if %fn_numb% LEQ 99 (
set formatted=  %fn_numb%
goto exitformat_engines)
if %fn_numb% LEQ 999 (
set formatted= %fn_numb%
goto exitformat_engines)
if %fn_numb% LEQ 9999 (
set formatted=%fn_numb%
goto exitformat_engines)
:exitformat_engines
EXIT /B 0

:format_engwins
set /A fn_numb=%~1
if %fn_numb% LEQ 9 (
set formatted=  %fn_numb%
goto exitformat_engwins)
if %fn_numb% LEQ 99 (
set formatted= %fn_numb%
goto exitformat_engwins)
if %fn_numb% LEQ 999 set formatted=%fn_numb%
:exitformat_engwins
EXIT /B 0

:StartTimer
set StartTIME=%TIME%
for /f "usebackq tokens=1-4 delims=:., " %%f in (`echo %StartTIME: =0%`) do set /a Start100S=1%%f*360000+1%%g*6000+1%%h*100+1%%i-36610100
EXIT /B 0

:StopTimer
set StopTIME=%TIME%
for /f "usebackq tokens=1-4 delims=:., " %%f in (`echo %StopTIME: =0%`) do set /a Stop100S=1%%f*360000+1%%g*6000+1%%h*100+1%%i-36610100
if %Stop100S% LSS %Start100S% set /a Stop100S+=8640000
set /a TookTime=%Stop100S%-%Start100S%
set TookTimePadded=0%TookTime%
EXIT /B 0

:StartTimer2
set StartTIME2=%TIME%
for /f "usebackq tokens=1-4 delims=:., " %%f in (`echo %StartTIME2: =0%`) do set /a Start100S2=1%%f*360000+1%%g*6000+1%%h*100+1%%i-36610100
EXIT /B 0

:StopTimer2
set StopTIME2=%TIME%
for /f "usebackq tokens=1-4 delims=:., " %%f in (`echo %StopTIME2: =0%`) do set /a Stop100S2=1%%f*360000+1%%g*6000+1%%h*100+1%%i-36610100
if %Stop100S2% LSS %Start100S2% set /a Stop100S2+=8640000
set /a TookTime2=%Stop100S2%-%Start100S2%
set TookTimePadded2=0%TookTime2%
EXIT /B 0

:DisplayTimerResult
set /A t_elapsed=%TookTime:~0,-2%.%TookTimePadded:~-2% 2>nul
set /A t_seconds=%t_elapsed%
set /A t_minutes=%t_seconds%/60
set /A t_hours=%t_minutes%/60
set /A t_seconds-=%t_minutes%*60
set /A t_minutes-=%t_hours%*60
if %t_seconds% LEQ 9 set t_seconds=0%t_seconds%
if %t_minutes% LEQ 9 set t_minutes=0%t_minutes%
if %t_hours% LEQ 9 set t_hours=0%t_hours%
@echo *** Elapsed time: %t_hours%:%t_minutes%:%t_seconds%
EXIT /B 0

:EstimatedTime
set /A t_elapsed=%TookTime2:~0,-2%.%TookTimePadded2:~-2% 2>nul
set /A t_time_per_engine=%t_elapsed%/%engine_counter
set /A t_engines_to_do=(%numb_engines%-%engine_counter%)+1
set /A t_estimated=%t_engines_to_do%*%t_time_per_engine%
set /A t_seconds=%t_estimated%
set /A t_minutes=%t_seconds%/60
set /A t_hours=%t_minutes%/60
set /A t_seconds-=%t_minutes%*60
set /A t_minutes-=%t_hours%*60
if %t_seconds% LEQ 9 set t_seconds=0%t_seconds%
if %t_minutes% LEQ 9 set t_minutes=0%t_minutes%
if %t_hours% LEQ 9 set t_hours=0%t_hours%
@echo *** Estimated time until finishing: %t_hours%:%t_minutes%:%t_seconds%
EXIT /B 0

:bigaverage
del gs* 2>nul
set big_gamefile=%~1
set /A splits=(%numb_all_onlywins%)/70000
if %splits% EQU 0 (
  CALL :moveaverage %big_gamefile%
  EXIT /B 0
)
@echo *********************************************************************************************
@echo *** The pgn-file with won games has to be splitted in several files, because it is too big
@echo *** for the summary-tool by N.Pollock, which calculates the move-average. This will need some 
@echo *** extra time, sorry...
set /A splitsinc=%splits%+1
gamesplit %big_gamefile% %splitsinc% >NUL
set /A big_counter=0
set /A collect_averages=0
:bigavg_loop
  CALL :format_gsfile %big_counter%
  CALL :moveaverage gs%formatted%.pgn
  set /A collect_averages+=%ma_moveaverage%
  set /A big_counter+=1
  if %big_counter% GTR %splits% goto exit_bigavg_loop
goto bigavg_loop
:exit_bigavg_loop
set /A ma_moveaverage=(%collect_averages%/%splitsinc%)
del gs* 2>nul
EXIT /B 0